@@ -166,16 +166,21 @@ def generate_tasks(layer_path, mip, roi_start, chunk_size, grid_size, queue_name
               type=str, required=True, help='the path of output volume.')
 @click.option('--max-ram-size', '-r',
               default=15, type=int, help='the maximum ram size (GB) of worker process.')
-@click.option('--patch-size', '-z',
+@click.option('--output-patch-size', '-z',
               type=int, required=True, nargs=3, help='output patch size.')
+@click.option('--input-patch-size', '-i',
+              type=int, default=None, nargs=3, callback=default_none,
+              help='input patch size.')
 @click.option('--channel-num', '-c',
-              type=int, default=1, help='output patch channel number. It is 3 for affinity map.')
+              type=int, default=1, 
+              help='output patch channel number. It is 3 for affinity map.')
 @click.option('--dtype', '-d', type=click.Choice(['uint8', 'float16', 'float32']), 
               default='float32', help='output numerical precision.')
-@click.option('--patch-overlap', '-o',
+@click.option('--output-patch-overlap', '-o',
               type=int, default=None, nargs=3, callback=default_none,
               help='overlap of patches. default is 50% overlap')
-@click.option('--crop-chunk-margin', '-c', type=int, nargs=3, default=None,
+@click.option('--crop-chunk-margin', '-c', 
+              type=int, nargs=3, default=None,
               callback=default_none, help='size of margin to be cropped.')
 @click.option('--mip', '-m', type=click.IntRange(min=0, max=3), default=0, 
               help='the output mip level (default is 0).')
@@ -196,8 +201,8 @@ def generate_tasks(layer_path, mip, roi_start, chunk_size, grid_size, queue_name
               help='voxel size or resolution of mip 0 image.')
 @generator
 def setup_env(volume_start, volume_stop, volume_size, layer_path, max_ram_size,
-              patch_size, channel_num, dtype, patch_overlap, crop_chunk_margin, 
-              mip, thumbnail_mip, max_mip,
+              output_patch_size, input_patch_size, channel_num, dtype, 
+              output_patch_overlap, crop_chunk_margin, mip, thumbnail_mip, max_mip,
               queue_name, visibility_timeout, thumbnail, encoding, voxel_size):
     """Prepare storage info files and produce tasks."""
     assert not (volume_stop is None and volume_size is None)
@@ -208,36 +213,40 @@ def setup_env(volume_start, volume_stop, volume_size, layer_path, max_ram_size,
     if isinstance(volume_size, tuple):
         volume_size = Vec(*volume_size)
 
+    if input_patch_size is None:
+        input_patch_size = output_patch_size
+
     if volume_size:
         assert volume_stop is None
         volume_stop = volume_start + volume_size
     else:
         volume_size = volume_stop - volume_start
-    print('\nvolume start: ', volume_start)
-    print('volume stop: ', volume_stop)
-    print('volume size: ', volume_size)
+    print('\noutput volume start: ', volume_start)
+    print('output volume stop: ', volume_stop)
+    print('output volume size: ', volume_size)
     
-    if patch_overlap is None:
+    if output_patch_overlap is None:
         # use 50% patch overlap in default
-        patch_overlap = tuple(s//2 for s in patch_size)
-    assert patch_overlap[1] == patch_overlap[2]
+        output_patch_overlap = tuple(s//2 for s in output_patch_size)
+    assert output_patch_overlap[1] == output_patch_overlap[2]
 
     if crop_chunk_margin is None:
-        crop_chunk_margin = patch_overlap
+        crop_chunk_margin = output_patch_overlap
     assert crop_chunk_margin[1] == crop_chunk_margin[2]
-    print('crop chunk margin: ', crop_chunk_margin)
+    print('output crop chunk margin: ', crop_chunk_margin)
     
     if thumbnail:
         # thumnail requires maximum mip level of 5
-        thumnail_mip = max(thumbnail_mip, 5)
+        thumbnail_mip = max(thumbnail_mip, 5)
     
-    patch_stride = tuple(s - o for s, o in zip(patch_size, patch_overlap))
+    patch_stride = tuple(s - o for s, o in zip(
+        output_patch_size, output_patch_overlap))
     # total number of voxels per patch in one stride
     patch_voxel_num = np.product( patch_stride )
     # use half of the maximum ram size to store output buffer
     ideal_total_patch_num = int(max_ram_size*1e9/2/4/channel_num/patch_voxel_num)
     # the xy size should be the same
-    assert patch_size[1] == patch_size[2]
+    assert output_patch_size[1] == output_patch_size[2]
     # compute the output chunk/block size in cloud storage
     # assume that the crop margin size is the same with the patch overlap
     patch_num_start = int(ideal_total_patch_num ** (1./3.)  / 2)
@@ -250,20 +259,23 @@ def setup_env(volume_start, volume_stop, volume_size, layer_path, max_ram_size,
     max_factor = 2**max_mip
     factor = 2**mip
     for pnxy in range(patch_num_start, patch_num_stop):
-        if (pnxy*patch_stride[2]+patch_overlap[2]-2*crop_chunk_margin[2]) % max_factor != 0:
+        if (pnxy * patch_stride[2] + output_patch_overlap[2] -
+                2 * crop_chunk_margin[2]) % max_factor != 0:
             continue
         # patch number in z
         for pnz in range(patch_num_start, patch_num_stop):
-            if (pnz*patch_stride[0]+patch_overlap[0]-2*crop_chunk_margin[0]) % factor != 0:
+            if (pnz * patch_stride[0] + output_patch_overlap[0] -
+                    2 * crop_chunk_margin[0]) % factor != 0:
                 continue
             current_cost = ( pnxy * pnxy * pnz / ideal_total_patch_num - 1) ** 2 #+ (pnxy / pnz - 1) ** 2
             if current_cost < cost:
                 cost = current_cost
                 patch_num = (pnz, pnxy, pnxy)
-                
-    print('\npatch size: ', patch_size)
-    print('patch overlap: ', patch_overlap)
-    print('patch stride: ', patch_stride)
+    
+    print('\ninput patch size: ', input_patch_size)
+    print('output patch size: ', output_patch_size)
+    print('output patch overlap: ', output_patch_overlap)
+    print('output patch stride: ', patch_stride)
     print('patch number: ', patch_num)
 
     assert mip>=0
@@ -271,17 +283,28 @@ def setup_env(volume_start, volume_stop, volume_size, layer_path, max_ram_size,
     block_factor = 2 ** block_mip
     
     output_chunk_size = tuple(n*s+o-2*c for n, s, o, c in zip(
-        patch_num, patch_stride, patch_overlap, crop_chunk_margin))
+        patch_num, patch_stride, output_patch_overlap, crop_chunk_margin))
+    
+    input_chunk_size = tuple(ocs + ccm*2 + ips - ops for ocs, ccm, ips, ops in zip(
+        output_chunk_size, crop_chunk_margin, 
+        input_patch_size, output_patch_size))
 
+    input_chunk_start = tuple(vs - ccm - (ips-ops)//2 for vs, ccm, ips, ops in zip(
+        volume_start, crop_chunk_margin, input_patch_size, output_patch_size))
+    
     block_size = (output_chunk_size[0]//factor, 
                   output_chunk_size[1]//block_factor,
                   output_chunk_size[2]//block_factor)
-    print('\noutput chunk size: ', output_chunk_size)
+    
+    print('\ninput chunk size: ', input_chunk_size)
+    print('input volume start: ', input_chunk_start)
+    print('output chunk size: ', output_chunk_size)
+    print('output volume start: ', volume_start)
     print('block size: ', block_size)
     print('RAM size of each block: ', 
           np.prod(output_chunk_size)/1024/1024/1024*4*channel_num, ' GB')
     print('voxel utilization: ', 
-          np.prod(output_chunk_size)/np.prod(patch_num)/np.prod(patch_size))
+          np.prod(output_chunk_size)/np.prod(patch_num)/np.prod(output_patch_size))
    
     if not state['dry_run']:
         print('\ncheck that we are not overwriting existing info file.')
@@ -1218,8 +1241,7 @@ def save(tasks, name, volume_path, input_chunk_name, upload_log, nproc, create_t
         if not task['skip']:
             # the time elapsed was recorded internally
             state['operators'][name](task[input_chunk_name],
-                                     log=task.get('log', {'timer': {}}),
-                                     output_bbox=task.get('bbox', None))
+                                     log=task.get('log', {'timer': {}}))
             task['output_volume_path'] = volume_path
         yield task
 
